/********************************************************************/
/* AgbMemoryMap.h */
/* AGB Memory Map Constants */
/* */
/* Copyright (C) 1999-2001 NINTENDO Co.,Ltd. */
/********************************************************************/


/*------------------------------------------------------------------*/
/* Memory Map */
/*------------------------------------------------------------------*/
.set BOOT_ROM , 		0x00000000 	/* Boot ROM */
.set BOOT_ROM_END , 		(BOOT_ROM + 0x4000)
.set EX_WRAM , 			0x02000000 	/* CPU External Work RAM */
.set EX_WRAM_END , 		(EX_WRAM + 0x40000)
.set CPU_WRAM , 		0x03000000 	/* CPU Internal Work RAM */
.set CPU_WRAM_END , 		(CPU_WRAM + 0x8000)
.set WRAM , 			EX_WRAM 	/* Entire Work RAM */
.set WRAM_END , 		CPU_WRAM_END
.set SOUND_AREA_ADDR_BUF , 	(CPU_WRAM_END - 0x10)	/*Sound Driver Work Address */
.set INTR_CHECK_BUF , 		(CPU_WRAM_END - 0x8)	/* Interrupt Check */
.set SOFT_RESET_DIRECT_BUF , 	(CPU_WRAM_END - 0x6)	/* SoftReset() Specify Return to */
.set INTR_VECTOR_BUF , 		(CPU_WRAM_END - 0x4)	/* Interrupt Branch Address */
.set REG_BASE , 		0x04000000 	/* Registers */
.set REG_END , 			(REG_BASE + 0x300)
.set PLTT , 			0x05000000 	/* Palette RAM */
.set PLTT_END , 		(PLTT + 0x400)
.set BG_PLTT , 			(PLTT + 0x0) 	/* BG Palette RAM */
.set BG_PLTT_END , 		(PLTT + 0x200)
.set OBJ_PLTT , 		(PLTT + 0x200) 	/* OBJ Palette RAM */
.set OBJ_PLTT_END , 		(PLTT + 0x400)
.set VRAM , 			0x06000000 	/* VRAM */
.set VRAM_END , 		(VRAM + 0x18000)
.set BG_VRAM , 			(VRAM + 0x0) 	/* BG Character/Screen RAM */
.set BG_BITMAP0_VRAM , 		(VRAM + 0x0) 	/* BG Bitmap 0 RAM */
.set BG_BITMAP1_VRAM , 		(VRAM + 0xa000) 	/* BG Bitmap 1 RAM */
.set OBJ_MODE0_VRAM , 		(VRAM + 0x10000) 	/* OBJ Character RAM */
.set OBJ_MODE1_VRAM , 		(VRAM + 0x10000)
.set OBJ_MODE2_VRAM , 		(VRAM + 0x10000)
.set OBJ_MODE3_VRAM , 		(VRAM + 0x14000)
.set OBJ_MODE4_VRAM , 		(VRAM + 0x14000)
.set OBJ_MODE5_VRAM , 		(VRAM + 0x14000)
.set OBJ_VRAM_END , 		(VRAM + 0x18000)
.set OAM , 			0x07000000 	/* OAM */
.set OAM_END , 			(OAM + 0x400)
.set ROM_BANK0 , 		0x08000000 	/* ROM Bank 0 */
.set ROM_BANK0_END , 		0x0a000000
.set ROM_BANK1 , 		0x0a000000 	/* ROM Bank 1 */
.set ROM_BANK1_END , 		0x0c000000
.set ROM_BANK2 , 		0x0c000000 	/* ROM Bank 2 */
.set ROM_BANK2_END , 		0x0e000000
.set RAM_BANK , 		0x0c000000 	/* RAM Bank */
.set RAM_BANK_END , 		0x10000000
.set FLASH_1M , 		0x09fe0000 	/* 1M Flash Memory */
.set FLASH_1M_END , 		0x0a000000
/*------------------------------------------------------------------*/
/* Memory Size */
/*------------------------------------------------------------------*/
.set BOOT_ROM_SIZE , 		0x4000 	/* Boot ROM */
.set EX_WRAM_SIZE , 		0x40000 	/* CPU External Work RAM */
.set CPU_WRAM_SIZE , 		0x8000 	/* CPU Internal Work RAM */
.set WRAM_SIZE , 		0x48000 	/* Entire Work RAM */
.set PLTT_SIZE , 		(2*256*2) 	/* Palette RAM */
.set BG_PLTT_SIZE , 		(2*256) 	/* BG Palette RAM */
.set OBJ_PLTT_SIZE , 		(2*256) 	/* OBJ Palette RAM */
.set VRAM_SIZE , 		0x18000 	/* VRAM */
.set BG_MODE0_VRAM_SIZE , 	0x10000 	/* BG Character/Screen */
.set BG_MODE1_VRAM_SIZE , 	0x10000
.set BG_MODE2_VRAM_SIZE , 	0x10000
.set BG_MODE3_VRAM_SIZE , 	0x14000 	/* BG Bitmap */
.set BG_MODE4_VRAM_SIZE , 	0x0a000
.set BG_MODE5_VRAM_SIZE , 	0x0a000
.set OBJ_MODE0_VRAM_SIZE , 	0x08000 	/* OBJ Character */
.set OBJ_MODE1_VRAM_SIZE , 	0x08000
.set OBJ_MODE2_VRAM_SIZE , 	0x08000
.set OBJ_MODE3_VRAM_SIZE , 	0x04000
.set OBJ_MODE4_VRAM_SIZE , 	0x04000
.set OBJ_MODE5_VRAM_SIZE , 	0x04000
.set OAM_SIZE , 		(8*128) 	/* OAM */
.set ROM_BANK_SIZE , 		0x02000000 	/* ROM Bank */
.set ROM_BANK0_SIZE , 		0x02000000 	/* ROM Bank 0 */
.set ROM_BANK1_SIZE , 		0x02000000 	/* ROM Bank 1 */
.set ROM_BANK2_SIZE , 		0x02000000 	/* ROM Bank 2 */
.set RAM_BANK_SIZE , 		0x02000000 	/* RAM Bank */
.set FLASH_1M_SIZE , 		0x20000 	/* 1M Flash Memory */
/*------------------------------------------------------------------*/
/* Register Address */
/*------------------------------------------------------------------*/
.set REG_IME , 			(REG_BASE + 0x208) 	/* Interrupt Master Enable */
.set REG_IE , 			(REG_BASE + 0x200) 	/* Interrupt Enable */
.set REG_IF , 			(REG_BASE + 0x202) 	/* Interrupt Request */
.set REG_WAITCNT , 		(REG_BASE + 0x204) 	/* Game Pak Wait Control */
.set REG_DISPCNT , 		(REG_BASE + 0x0) 	/* Display Control */
.set REG_STAT , 		(REG_BASE + 0x4) 	/* Status */
.set REG_VCOUNT , 		(REG_BASE + 0x6) 	/* V Counter */
.set REG_MOSAIC , 		(REG_BASE + 0x4c) 	/* Mosaic Size */
.set REG_BLDCNT , 		(REG_BASE + 0x50) 	/* Blend Mode Control */
.set REG_BLDALPHA , 		(REG_BASE + 0x52) 	/* Semi-transparent Parameter */
.set REG_BLDY , 		(REG_BASE + 0x54) 	/* Brightness Change Parameter */
.set REG_WINCNT , 		(REG_BASE + 0x40) 	/* Window Control */
.set REG_WIN0H , 		(REG_BASE + 0x40) 	/* Window 0 Horizontal Area */
.set REG_WIN1H , 		(REG_BASE + 0x42) 	/* Window 1 Horizontal Area */
.set REG_WIN0V , 		(REG_BASE + 0x44) 	/* Window 0 Vertical Area */
.set REG_WIN1V , 		(REG_BASE + 0x46) 	/* WIndow 1 Vertical Area */
.set REG_WININ , 		(REG_BASE + 0x48) 	/* Internal Window Control */
.set REG_WIN0 , 		(REG_BASE + 0x48) 	/* Window 0 Control */
.set REG_WIN1 , 		(REG_BASE + 0x49) 	/* Window 1 Control */
.set REG_WINOUT , 		(REG_BASE + 0x4a) 	/* External Window Control */
.set REG_OBJWIN , 		(REG_BASE + 0x4b) 	/* OBJ Window Control */
.set REG_BGCNT , 		(REG_BASE + 0x8) 	/* BG Control */
.set REG_BG0CNT , 		(REG_BASE + 0x8) 	/* BG 0 Control */
.set REG_BG1CNT , 		(REG_BASE + 0xa) 	/* BG 1 Control */
.set REG_BG2CNT , 		(REG_BASE + 0xc) 	/* BG 2 Control */
.set REG_BG3CNT , 		(REG_BASE + 0xe) 	/* BG 3 Control */
.set REG_BGOFS , 		(REG_BASE + 0x10) 	/* BG Offset */
.set REG_BG0HOFS , 		(REG_BASE + 0x10) 	/* BG 0 H Offset */
.set REG_BG0VOFS , 		(REG_BASE + 0x12) 	/* BG 0 V Offset */
.set REG_BG1HOFS , 		(REG_BASE + 0x14) 	/* BG 1 H Offset */
.set REG_BG1VOFS , 		(REG_BASE + 0x16) 	/* BG 1 V Offset */
.set REG_BG2HOFS , 		(REG_BASE + 0x18) 	/* BG 2 H Offset */
.set REG_BG2VOFS , 		(REG_BASE + 0x1a) 	/* BG 2 V Offset */
.set REG_BG3HOFS , 		(REG_BASE + 0x1c) 	/* BG 3 H Offset */
.set REG_BG3VOFS , 		(REG_BASE + 0x1e) 	/* BG 3 V Offset */
.set REG_BG2AFFINE , 		(REG_BASE + 0x20) 	/* BG 2 Affin Transformation Parameters */
.set REG_BG2PA , 		(REG_BASE + 0x20) 	/* BG 2 Line Direction X Coordinate Difference */
.set REG_BG2PB , 		(REG_BASE + 0x22) 	/* BG 3 Vertical Direction X Coordinate Difference */
.set REG_BG2PC , 		(REG_BASE + 0x24) 	/* BG 2 Line Direction Y coordinate Difference */
.set REG_BG2PD , 		(REG_BASE + 0x26) 	/* BG 2 Vertical Direction Y Coordinate Difference */
.set REG_BG2X , 		(REG_BASE + 0x28) 	/* BG 2 Start X Coordinate */
.set REG_BG2X_L , 		(REG_BASE + 0x28)
.set REG_BG2X_H , 		(REG_BASE + 0x2a)
.set REG_BG2Y , 		(REG_BASE + 0x2c) 	/* BG 2 Start Y Coordinate */
.set REG_BG2Y_L , 		(REG_BASE + 0x2c)
.set REG_BG2Y_H , 		(REG_BASE + 0x2e)
.set REG_BG3AFFINE , 		(REG_BASE + 0x30) 	/* BG 3 Affin Transformation Parameters */
.set REG_BG3PA , 		(REG_BASE + 0x30) 	/* BG 3 Line Direction X Coordinate Difference */
.set REG_BG3PB , 		(REG_BASE + 0x32) 	/* BG 3 Vertical Direction X Coordinate Difference */
.set REG_BG3PC , 		(REG_BASE + 0x34) 	/* BG 3 Line Direction Y Coordinate Difference */
.set REG_BG3PD , 		(REG_BASE + 0x36) 	/* BG 3 Vertical Direction Y Coordinate Difference */
.set REG_BG3X , 		(REG_BASE + 0x38) 	/* BG 3 Start X Coordinate */
.set REG_BG3X_L , 		(REG_BASE + 0x38)
.set REG_BG3X_H , 		(REG_BASE + 0x3a)
.set REG_BG3Y , 		(REG_BASE + 0x3c) 	/* BG 3 Start Y Coordinate */
.set REG_BG3Y_L , 		(REG_BASE + 0x3c)
.set REG_BG3Y_H , 		(REG_BASE + 0x3e)
.set REG_SOUNDCNT , 		(REG_BASE + 0x80) 	/* Sound Control */
.set REG_SOUNDCNT_L , 		(REG_BASE + 0x80)
.set REG_SOUNDCNT_H , 		(REG_BASE + 0x82)
.set REG_SOUNDCNT_X , 		(REG_BASE + 0x84)
.set REG_SOUNDBIAS , 		(REG_BASE + 0x88) 	/* Sound BIAS */
.set REG_SOUND1CNT , 		(REG_BASE + 0x60) 	/* Sound 1 Control */
.set REG_SOUND1CNT_L , 		(REG_BASE + 0x60)
.set REG_SOUND1CNT_H , 		(REG_BASE + 0x62)
.set REG_SOUND1CNT_X , 		(REG_BASE + 0x64)
.set REG_SOUND2CNT , 		(REG_BASE + 0x68) 	/* Sound 2 Control */
.set REG_SOUND2CNT_L , 		(REG_BASE + 0x68)
.set REG_SOUND2CNT_H , 		(REG_BASE + 0x6c)
.set REG_SOUND3CNT , 		(REG_BASE + 0x70) 	/* Sound 3 Control */
.set REG_SOUND3CNT_L , 		(REG_BASE + 0x70)
.set REG_SOUND3CNT_H , 		(REG_BASE + 0x72)
.set REG_SOUND3CNT_X , 		(REG_BASE + 0x74)
.set REG_SOUND4CNT , 		(REG_BASE + 0x78) 	/* Sound 4 Control */
.set REG_SOUND4CNT_L , 		(REG_BASE + 0x78)
.set REG_SOUND4CNT_H , 		(REG_BASE + 0x7c)
.set REG_WAVE_RAM , 		(REG_BASE + 0x90) 	/* Sound 3 Waveform RAM */
.set REG_WAVE_RAM0 , 		(REG_BASE + 0x90)
.set REG_WAVE_RAM0_L , 		(REG_BASE + 0x90)
.set REG_WAVE_RAM0_H , 		(REG_BASE + 0x92)
.set REG_WAVE_RAM1 , 		(REG_BASE + 0x94)
.set REG_WAVE_RAM1_L , 		(REG_BASE + 0x94)
.set REG_WAVE_RAM1_H , 		(REG_BASE + 0x96)
.set REG_WAVE_RAM2 , 		(REG_BASE + 0x98)
.set REG_WAVE_RAM2_L , 		(REG_BASE + 0x98)
.set REG_WAVE_RAM2_H , 		(REG_BASE + 0x9a)
.set REG_WAVE_RAM3 , 		(REG_BASE + 0x9c)
.set REG_WAVE_RAM3_L , 		(REG_BASE + 0x9c)
.set REG_WAVE_RAM3_H , 		(REG_BASE + 0x9e)
.set REG_FIFO , 		(REG_BASE + 0xa0) 	/* Sound FIFO */
.set REG_FIFO_A , 		(REG_BASE + 0xa0)
.set REG_FIFO_A_L , 		(REG_BASE + 0xa0)
.set REG_FIFO_A_H , 		(REG_BASE + 0xa2)
.set REG_FIFO_B , 		(REG_BASE + 0xa4)
.set REG_FIFO_B_L , 		(REG_BASE + 0xa4)
.set REG_FIFO_B_H , 		(REG_BASE + 0xa6)
.set REG_DMA0 , 		(REG_BASE + 0xb0) 	/* DMA 0 */
.set REG_DMA0SAD , 		(REG_BASE + 0xb0) 	/* (Internal Memory -> */
	/* Internal Memory) */
.set REG_DMA0SAD_L , 		(REG_BASE + 0xb0)
.set REG_DMA0SAD_H , 		(REG_BASE + 0xb2)
.set REG_DMA0DAD , 		(REG_BASE + 0xb4)
.set REG_DMA0DAD_L , 		(REG_BASE + 0xb4)
.set REG_DMA0DAD_H , 		(REG_BASE + 0xb6)
.set REG_DMA0CNT , 		(REG_BASE + 0xb8)
.set REG_DMA0CNT_L , 		(REG_BASE + 0xb8)
.set REG_DMA0CNT_H , 		(REG_BASE + 0xba)
.set REG_DMA1 , 		(REG_BASE + 0xbc) 	/* DMA 1 */
.set REG_DMA1SAD , 		(REG_BASE + 0xbc) 	/* (Internal Memory -> Sound */
	/* FIFO/Internal Memory) */
.set REG_DMA1SAD_L , 		(REG_BASE + 0xbc)
.set REG_DMA1SAD_H , 		(REG_BASE + 0xbe)
.set REG_DMA1DAD , 		(REG_BASE + 0xc0)
.set REG_DMA1DAD_L , 		(REG_BASE + 0xc0)
.set REG_DMA1DAD_H , 		(REG_BASE + 0xc2)
.set REG_DMA1CNT , 		(REG_BASE + 0xc4)
.set REG_DMA1CNT_L , 		(REG_BASE + 0xc4)
.set REG_DMA1CNT_H , 		(REG_BASE + 0xc6)
.set REG_DMA2 , 		(REG_BASE + 0xc8) 	/* DMA 2 */
.set REG_DMA2SAD , 		(REG_BASE + 0xc8) 	/* (Internal Memory -> Sound */
	/* FIFO/Internal Memory) */
.set REG_DMA2SAD_L , 		(REG_BASE + 0xc8)
.set REG_DMA2SAD_H , 		(REG_BASE + 0xca)
.set REG_DMA2DAD , 		(REG_BASE + 0xcc)
.set REG_DMA2DAD_L , 		(REG_BASE + 0xcc)
.set REG_DMA2DAD_H , 		(REG_BASE + 0xce)
.set REG_DMA2CNT , 		(REG_BASE + 0xd0)
.set REG_DMA2CNT_L , 		(REG_BASE + 0xd0)
.set REG_DMA2CNT_H , 		(REG_BASE + 0xd2)
.set REG_DMA3 , 		(REG_BASE + 0xd4) 	/* DMA 3 */
.set REG_DMA3SAD , 		(REG_BASE + 0xd4) 	/* (Game Pak/Internal Memory <--> */
	/* Internal Memory) */
.set REG_DMA3SAD_L , 		(REG_BASE + 0xd4)
.set REG_DMA3SAD_H , 		(REG_BASE + 0xd6)
.set REG_DMA3DAD , 		(REG_BASE + 0xd8)
.set REG_DMA3DAD_L , 		(REG_BASE + 0xd8)
.set REG_DMA3DAD_H , 		(REG_BASE + 0xda)
.set REG_DMA3CNT , 		(REG_BASE + 0xdc)
.set REG_DMA3CNT_L , 		(REG_BASE + 0xdc)
.set REG_DMA3CNT_H , 		(REG_BASE + 0xde)
.set REG_TM0CNT , 		(REG_BASE + 0x100) 	/* Timer 0 */
.set REG_TM0CNT_L , 		(REG_BASE + 0x100)
.set REG_TM0CNT_H , 		(REG_BASE + 0x102)
.set REG_TM1CNT , 		(REG_BASE + 0x104) 	/* Timer 1 */
.set REG_TM1CNT_L , 		(REG_BASE + 0x104)
.set REG_TM1CNT_H , 		(REG_BASE + 0x106)
.set REG_TM2CNT , 		(REG_BASE + 0x108) 	/* Timer 2 */
.set REG_TM2CNT_L , 		(REG_BASE + 0x108)
.set REG_TM2CNT_H , 		(REG_BASE + 0x10a)
.set REG_TM3CNT , 		(REG_BASE + 0x10c) 	/* Timer 3 */
.set REG_TM3CNT_L , 		(REG_BASE + 0x10c)
.set REG_TM3CNT_H , 		(REG_BASE + 0x10e)
.set REG_RCNT , 		(REG_BASE + 0x134) 	/* General Input/Output Control */
.set REG_SIOCNT , 		(REG_BASE + 0x128) 	/* Serial Communication Control */
.set REG_SIODATA8 , 		(REG_BASE + 0x12a) 	/* 8bit Serial Communication Data */
.set REG_SIODATA32 , 		(REG_BASE + 0x120) 	/* 32bit Serial Communication Data */
.set REG_SIOMLT_SEND , 		(REG_BASE + 0x12a) 	/* Multi-play SIO Send Data */
.set REG_SIOMLT_RECV , 		(REG_BASE + 0x120) 	/* Multi-play SIO Receive Data */
.set REG_SIOMULTI0 , 		(REG_BASE + 0x120)
.set REG_SIOMULTI1 , 		(REG_BASE + 0x122)
.set REG_SIOMULTI2 , 		(REG_BASE + 0x124)
.set REG_SIOMULTI3 , 		(REG_BASE + 0x126)
.set REG_JOYCNT , 		(REG_BASE + 0x140) 	/* JOY Bus Communication Control */
.set REG_JOYSTAT , 		(REG_BASE + 0x158) 	/* JOY Bus Status */
.set REG_JOY_RECV , 		(REG_BASE + 0x150) 	/* JOY Bus Receive Data */
.set REG_JOY_RECV_L , 		(REG_BASE + 0x150)
.set REG_JOY_RECV_H , 		(REG_BASE + 0x152)
.set REG_JOY_TRANS , 		(REG_BASE + 0x154) 	/* JOY Bus Send Data */
.set REG_JOY_TRANS_L , 		(REG_BASE + 0x154)
.set REG_JOY_TRANS_H , 		(REG_BASE + 0x156)
.set REG_KEYINPUT , 		(REG_BASE + 0x130) 	/* Key Input */
.set REG_KEYCNT , 		(REG_BASE + 0x132) 	/* Key Control */
/*------------------------------------------------------------------*/
/* Register Offset */
/*------------------------------------------------------------------*/
.set OFFSET_REG_IME , 		0x208 	/* Interrupt Master Enable */
.set OFFSET_REG_IE , 		0x200 	/* Interrupt Enable */
.set OFFSET_REG_IF , 		0x202 	/* Interrupt Request */
.set OFFSET_REG_WAITCNT , 	0x204 	/* Game Pak Wait Control */
.set OFFSET_REG_DISPCNT , 	0x0 	/* Display Control */
.set OFFSET_REG_STAT , 		0x4 	/* Status */
.set OFFSET_REG_VCOUNT , 	0x6 	/* V Counter */
.set OFFSET_REG_MOSAIC , 	0x4c 	/* Mosaic Size */
.set OFFSET_REG_BLDCNT , 	0x50 	/* Blend Mode Control */
.set OFFSET_REG_BLDALPHA , 	0x52 	/* Semi-trams[aremt Parameter */
.set OFFSET_REG_BLDY , 		0x54 	/* Brightness Change Parameter */
.set OFFSET_REG_WINCNT , 	0x40 	/* Window Control */
.set OFFSET_REG_WIN0H , 	0x40 	/* Window 0 Horizontal Area */
.set OFFSET_REG_WIN1H , 	0x42 	/* Window 1 Horizontal Area */
.set OFFSET_REG_WIN0V , 	0x44 	/* Window 0 Vertical Area */
.set OFFSET_REG_WIN1V , 	0x46 	/* Window 1 Vertical Area */
.set OFFSET_REG_WININ , 	0x48 	/* Internal Window Control */
.set OFFSET_REG_WIN0 , 		0x48 	/* Window 0 Control */
.set OFFSET_REG_WIN1 , 		0x49 	/* Window 1 Control */
.set OFFSET_REG_WINOUT , 	0x4a 	/* External Window Control */
.set OFFSET_REG_OBJWIN , 	0x4b 	/* OBJ Window Control */
.set OFFSET_REG_BGCNT , 	0x8 	/* BG Control */
.set OFFSET_REG_BG0CNT , 	0x8 	/* BG 0 Control */
.set OFFSET_REG_BG1CNT , 	0xa 	/* BG 1 Control */
.set OFFSET_REG_BG2CNT , 	0xc 	/* BG 2 Control */
.set OFFSET_REG_BG3CNT , 	0xe 	/* BG 3 Control */
.set OFFSET_REG_BGOFS , 	0x10 	/* BG Offset */
.set OFFSET_REG_BG0HOFS , 	0x10 	/* BG 0 H Offset */
.set OFFSET_REG_BG0VOFS , 	0x12 	/* BG 0 V Offset */
.set OFFSET_REG_BG1HOFS , 	0x14 	/* BG 1 H Offset */
.set OFFSET_REG_BG1VOFS , 	0x16 	/* BG 1 V Offset */
.set OFFSET_REG_BG2HOFS , 	0x18 	/* BG 2 H Offset */
.set OFFSET_REG_BG2VOFS , 	0x1a 	/* BG 2 V Offset */
.set OFFSET_REG_BG3HOFS , 	0x1c 	/* BG 3 H Offset */
.set OFFSET_REG_BG3VOFS , 	0x1e 	/* BG 3 V Offset */
.set OFFSET_REG_BG2AFFINE , 	0x20 	/* BG 2 Affin Transformation Parameters */
.set OFFSET_REG_BG2PA , 	0x20 	/* BG 2 Line Direction X Coordinate Difference */
.set OFFSET_REG_BG2PB , 	0x22 	/* BG 2 Vertical Direction X Coordinate Difference */
.set OFFSET_REG_BG2PC , 	0x24 	/* BG 2 Line Direction Y Coordinate Difference */
.set OFFSET_REG_BG2PD , 	0x26 	/* BG 2 Vertical Direction Y Coordinate Difference */
.set OFFSET_REG_BG2X , 		0x28 	/* BG 2 Start X Coordinate */
.set OFFSET_REG_BG2X_L , 	0x28
.set OFFSET_REG_BG2X_H , 	0x2a
.set OFFSET_REG_BG2Y , 		0x2c 	/* BG 2 Start Y Coordinate */
.set OFFSET_REG_BG2Y_L , 	0x2c
.set OFFSET_REG_BG2Y_H , 	0x2e
.set OFFSET_REG_BG3AFFINE , 	0x30 	/* BG 3 Affin Transformation Parameters */
.set OFFSET_REG_BG3PA , 	0x30 	/* BG 3 Line Direction X Coordinate Difference */
.set OFFSET_REG_BG3PB , 	0x32 	/* BG 3 Vertical Direction X Coordinate Difference */
.set OFFSET_REG_BG3PC , 	0x34 	/* BG 3 Line Direction Y Coordinate Difference */
.set OFFSET_REG_BG3PD , 	0x36 	/* BG 3 Vertical Direction Y Coordinate Difference */
.set OFFSET_REG_BG3X , 		0x38 	/* BG 3 Start X Coordinate */
.set OFFSET_REG_BG3X_L , 	0x38
.set OFFSET_REG_BG3X_H , 	0x3a
.set OFFSET_REG_BG3Y , 		0x3c 	/* BG 3 Start Y Coordinate */
.set OFFSET_REG_BG3Y_L , 	0x3c
.set OFFSET_REG_BG3Y_H , 	0x3e
.set OFFSET_REG_SOUNDCNT , 	0x80 	/* Sound Control */
.set OFFSET_REG_SOUNDCNT_L , 	0x80
.set OFFSET_REG_SOUNDCNT_H , 	0x82
.set OFFSET_REG_SOUNDCNT_X , 	0x84
.set OFFSET_REG_SOUNDBIAS , 	0x88 	/* Sound BIAS */
.set OFFSET_REG_SOUND1CNT , 	0x60 	/* Sound 1 Control */
.set OFFSET_REG_SOUND1CNT_L , 	0x60
.set OFFSET_REG_SOUND1CNT_H , 	0x62
.set OFFSET_REG_SOUND1CNT_X , 	0x64
.set OFFSET_REG_SOUND2CNT , 	0x68 	/* Sound 2 Control */
.set OFFSET_REG_SOUND2CNT_L , 	0x68
.set OFFSET_REG_SOUND2CNT_H , 	0x6c
.set OFFSET_REG_SOUND3CNT , 	0x70 	/* Sound 3 Control */
.set OFFSET_REG_SOUND3CNT_L , 	0x70
.set OFFSET_REG_SOUND3CNT_H , 	0x72
.set OFFSET_REG_SOUND3CNT_X , 	0x74
.set OFFSET_REG_SOUND4CNT , 	0x78 	/* Sound 4 Control */
.set OFFSET_REG_SOUND4CNT_L , 	0x78
.set OFFSET_REG_SOUND4CNT_H , 	0x7c
.set OFFSET_REG_WAVE_RAM , 	0x90 	/* Sound 3 Waveform RAM */
.set OFFSET_REG_WAVE_RAM0 , 	0x90
.set OFFSET_REG_WAVE_RAM0_L , 	0x90
.set OFFSET_REG_WAVE_RAM0_H , 	0x92
.set OFFSET_REG_WAVE_RAM1 , 	0x94
.set OFFSET_REG_WAVE_RAM1_L , 	0x94
.set OFFSET_REG_WAVE_RAM1_H , 	0x96
.set OFFSET_REG_WAVE_RAM2 , 	0x98
.set OFFSET_REG_WAVE_RAM2_L , 	0x98
.set OFFSET_REG_WAVE_RAM2_H , 	0x9a
.set OFFSET_REG_WAVE_RAM3 , 	0x9c
.set OFFSET_REG_WAVE_RAM3_L , 	0x9c
.set OFFSET_REG_WAVE_RAM3_H , 	0x9e
.set OFFSET_REG_FIFO , 		0xa0 	/* Sound FIFO */
.set OFFSET_REG_FIFO_A , 	0xa0
.set OFFSET_REG_FIFO_A_L , 	0xa0
.set OFFSET_REG_FIFO_A_H , 	0xa2
.set OFFSET_REG_FIFO_B , 	0xa4
.set OFFSET_REG_FIFO_B_L , 	0xa4
.set OFFSET_REG_FIFO_B_H , 	0xa6
.set OFFSET_REG_DMA0 , 		0xb0 	/* DMA 0 */
.set OFFSET_REG_DMA0SAD , 	0xb0 	/* (Internal Memory -> Internal */
	/* Memory) */
.set OFFSET_REG_DMA0SAD_L , 	0xb0
.set OFFSET_REG_DMA0SAD_H , 	0xb2
.set OFFSET_REG_DMA0DAD , 	0xb4
.set OFFSET_REG_DMA0DAD_L , 	0xb4
.set OFFSET_REG_DMA0DAD_H , 	0xb6
.set OFFSET_REG_DMA0CNT , 	0xb8
.set OFFSET_REG_DMA0CNT_L , 	0xb8
.set OFFSET_REG_DMA0CNT_H , 	0xba
.set OFFSET_REG_DMA1 , 		0xbc 	/* DMA 1 */
.set OFFSET_REG_DMA1SAD , 	0xbc 	/* (Internal Memory -> Sound */
	/* FIFO/Internal Memory) */
.set OFFSET_REG_DMA1SAD_L , 	0xbc
.set OFFSET_REG_DMA1SAD_H , 	0xbe
.set OFFSET_REG_DMA1DAD , 	0xc0
.set OFFSET_REG_DMA1DAD_L , 	0xc0
.set OFFSET_REG_DMA1DAD_H , 	0xc2
.set OFFSET_REG_DMA1CNT , 	0xc4
.set OFFSET_REG_DMA1CNT_L , 	0xc4
.set OFFSET_REG_DMA1CNT_H , 	0xc6
.set OFFSET_REG_DMA2 , 		0xc8 	/* DMA 2 */
.set OFFSET_REG_DMA2SAD , 	0xc8 	/* (Internal Memory -> Sound */
	/* FIFO/Internal Memory) */
.set OFFSET_REG_DMA2SAD_L , 	0xc8
.set OFFSET_REG_DMA2SAD_H , 	0xca
.set OFFSET_REG_DMA2DAD , 	0xcc
.set OFFSET_REG_DMA2DAD_L , 	0xcc
.set OFFSET_REG_DMA2DAD_H , 	0xce
.set OFFSET_REG_DMA2CNT , 	0xd0
.set OFFSET_REG_DMA2CNT_L , 	0xd0
.set OFFSET_REG_DMA2CNT_H , 	0xd2
.set OFFSET_REG_DMA3 , 		0xd4 	/* DMA 3 */
.set OFFSET_REG_DMA3SAD , 	0xd4 	/* (Game Pak/Internal Memory <--> */
	/* Internal Memory) */
.set OFFSET_REG_DMA3SAD_L , 	0xd4
.set OFFSET_REG_DMA3SAD_H , 	0xd6
.set OFFSET_REG_DMA3DAD , 	0xd8
.set OFFSET_REG_DMA3DAD_L , 	0xd8
.set OFFSET_REG_DMA3DAD_H , 	0xda
.set OFFSET_REG_DMA3CNT , 	0xdc
.set OFFSET_REG_DMA3CNT_L , 	0xdc
.set OFFSET_REG_DMA3CNT_H , 	0xde
.set OFFSET_REG_TM0CNT , 	0x100 	/* Timer 0 */
.set OFFSET_REG_TM0CNT_L , 	0x100
.set OFFSET_REG_TM0CNT_H , 	0x102
.set OFFSET_REG_TM1CNT , 	0x104 	/* Timer 1 */
.set OFFSET_REG_TM1CNT_L , 	0x104
.set OFFSET_REG_TM1CNT_H , 	0x106
.set OFFSET_REG_TM2CNT , 	0x108 	/* Timer 2 */
.set OFFSET_REG_TM2CNT_L , 	0x108
.set OFFSET_REG_TM2CNT_H , 	0x10a
.set OFFSET_REG_TM3CNT , 	0x10c 	/* Timer 3 */
.set OFFSET_REG_TM3CNT_L , 	0x10c
.set OFFSET_REG_TM3CNT_H , 	0x10e
.set OFFSET_REG_RCNT , 		0x134 	/* General Input/Output Control */
.set OFFSET_REG_SIOCNT , 	0x128 	/* Serial Communication Control */
.set OFFSET_REG_SIODATA8 , 	0x12a 	/* 8 bit Serial Communication Data */
.set OFFSET_REG_SIODATA32 , 	0x120 	/* 32 bit Serial Communication Data */
.set OFFSET_REG_SIOMLT_SEND , 	0x12a 	/* Multi-play SIO Send Data */
.set OFFSET_REG_SIOMLT_RECV , 	0x120 	/* Multi-play SIO Receive Data */
.set OFFSET_REG_SIOMULTI0 , 	0x120
.set OFFSET_REG_SIOMULTI1 , 	0x122
.set OFFSET_REG_SIOMULTI2 , 	0x124
.set OFFSET_REG_SIOMULTI3 , 	0x126
.set OFFSET_REG_JOYCNT , 	0x140 	/* JOY Bus Communication Control */
.set OFFSET_REG_JOYSTAT , 	0x158 	/* JOY Bus Status */
.set OFFSET_REG_JOY_RECV , 	0x150 	/* JOY Bus Receive Data */
.set OFFSET_REG_JOY_RECV_L , 	0x150
.set OFFSET_REG_JOY_RECV_H , 	0x152
.set OFFSET_REG_JOY_TRANS , 	0x154 	/* JOY Bus Send Data */
.set OFFSET_REG_JOY_TRANS_L , 	0x154
.set OFFSET_REG_JOY_TRANS_H , 	0x156
.set OFFSET_REG_KEYINPUT , 	0x130 	/* Key Input */
.set OFFSET_REG_KEYCNT , 	0x132 	/* Key Control */
